@@ -162,6 +162,7 @@ FILTER(a, x -> x.a.b.c.d.e.f.g)
 FILTER(a, x -> FOO(x.a.b.c.d.e.f.g) + x.a.b.c.d.e.f.g)
 TIMESTAMP_FROM_PARTS(2019, 1, 10, 2, 3, 4, 123456789, 'America/Los_Angeles')
 TIMESTAMPDIFF(CURRENT_TIMESTAMP(), 1, DAY)
+TIME_FROM_PARTS(14, 30, 45, 123456789)
 DATETIME_DIFF(CURRENT_DATE, 1, DAY)
 QUANTILE(x, 0.5)
 REGEXP_REPLACE('new york', '(\w)(\w*)', x -> UPPER(x[1]) || LOWER(x[2]))
@@ -176,6 +177,8 @@ REPLACE('new york', ' ')
 DATE(x) = DATE(y)
 TIMESTAMP(DATE(x))
 TIMESTAMP_TRUNC(COALESCE(time_field, CURRENT_TIMESTAMP()), DAY)
+MONTHNAME(x)
+MONTHS_BETWEEN(CAST('2019-03-15' AS DATE), CAST('2019-02-15' AS DATE))
 COUNT(DISTINCT CASE WHEN DATE_TRUNC('ISOWEEK', DATE(time_field)) = DATE_TRUNC('ISOWEEK', DATE(time_field2)) THEN report_id ELSE NULL END)
 COUNT(a, b)
 x[y - 1]
@@ -922,6 +925,7 @@ SELECT COT(x)
 SELECT COTH(x)
 SELECT CSC(x)
 SELECT CSCH(x)
+SELECT RADIANS(x)
 SELECT SEC(x)
 SELECT SECH(x)
 SELECT UTC_DATE
@@ -942,4 +946,5 @@ SELECT (LEAD(foo1, 1, 0)) OVER (PARTITION BY foo2 ORDER BY foo3) FROM t
 SELECT LAST_VALUE(CASE WHEN interval <> 'foo' THEN interval END) IGNORE NULLS FROM t
 WITH UNNEST AS (SELECT 1 AS UNNEST) SELECT UNNEST FROM UNNEST
 SELECT * FROM tbl GROUP BY GROUPING SETS ((a + 1, b * 1), c, CUBE (a, b), ROLLUP (c, d), (a + y, b * 1), ())
-SELECT * FROM tbl GROUP BY GROUPING SETS (GROUPING SETS (course), GROUPING SETS (type), CUBE (a), ROLLUP (b))
\ No newline at end of file
+SELECT * FROM tbl GROUP BY GROUPING SETS (GROUPING SETS (course), GROUPING SETS (type), CUBE (a), ROLLUP (b))
+SELECT analyze FROM (SELECT 1 AS analyze)
