@@ -52,6 +52,9 @@ FALSE;
 'x' = 'x';
 TRUE;
 
+STRUCT(NULL AS a);
+STRUCT(NULL AS a);
+
 NULL AND TRUE;
 NULL;
 
@@ -82,6 +85,33 @@ NULL;
 NULL = NULL;
 NULL;
 
+SELECT (EXISTS(SELECT 1 WHERE FALSE)) AND NULL;
+SELECT EXISTS(SELECT 1 WHERE FALSE) AND NULL;
+
+SELECT NULL AND (EXISTS(SELECT 1 WHERE FALSE));
+SELECT EXISTS(SELECT 1 WHERE FALSE) AND NULL;
+
+1 AND 0;
+FALSE;
+
+0 AND 1;
+FALSE;
+
+0 OR 1;
+TRUE;
+
+0 OR NULL;
+NULL;
+
+NULL OR 0;
+NULL;
+
+0 AND NULL;
+FALSE;
+
+NULL AND 0;
+FALSE;
+
 -- Can't optimize this because different engines do different things
 -- mysql converts to 0 and 1 but tsql does true and false
 NULL <=> NULL;
@@ -102,6 +132,27 @@ a AND b;
 a AND (b AND b);
 a AND b;
 
+-- bigquery doesn't allow unparenthesis comparisons
+(x is not null) != (y is null);
+(NOT x IS NULL) <> (y IS NULL);
+
+# dialect: mysql
+A XOR A;
+A XOR A;
+
+# dialect: mysql
+SELECT DISTINCT GREATEST(EXISTS(SELECT 1 WHERE FALSE), (EXISTS(SELECT 1 WHERE FALSE)) XOR ((0.08) IN ((t1.c0) XOR (t1.c0)))) AS ref0 FROM (SELECT NULL AS c0 UNION ALL SELECT 1 AS c0) AS t1, (SELECT 0.01 AS c1) AS t0;
+SELECT DISTINCT GREATEST(EXISTS(SELECT 1 WHERE FALSE), 0.08 IN (t1.c0 XOR t1.c0) XOR EXISTS(SELECT 1 WHERE FALSE)) AS ref0 FROM (SELECT NULL AS c0 UNION ALL SELECT 1 AS c0) AS t1, (SELECT 0.01 AS c1) AS t0;
+
+TRUE AND TRUE OR TRUE AND FALSE;
+TRUE;
+
+COALESCE(x, y) <> ALL (SELECT z FROM w);
+COALESCE(x, y) <> ALL (SELECT z FROM w);
+
+SELECT NOT (2 <> ALL (SELECT 2 UNION ALL SELECT 3));
+SELECT 2 = ANY(SELECT 2 UNION ALL SELECT 3);
+
 --------------------------------------
 -- Absorption
 --------------------------------------
@@ -147,6 +198,12 @@ A OR C;
 A AND (B AND C) AND (D AND E);
 A AND B AND C AND D AND E;
 
+A AND (A OR B) AND (A OR B OR C);
+A;
+
+(A OR B) AND (A OR C) AND (A OR B OR C);
+(A OR B) AND (A OR C);
+
 --------------------------------------
 -- Elimination
 --------------------------------------
@@ -183,6 +240,15 @@ NOT A;
 E OR (A AND B) OR C OR D OR (A AND NOT B);
 A OR C OR D OR E;
 
+(A AND B) OR (A AND NOT B) OR (A AND NOT B);
+A;
+
+(A AND B) OR (A AND B) OR (A AND NOT B);
+A;
+
+(A AND B) OR (A AND NOT B) OR (A AND B) OR (A AND NOT B);
+A;
+
 --------------------------------------
 -- Associativity
 --------------------------------------
@@ -225,6 +291,13 @@ x - 1;
 A AND D AND B AND E AND F AND G AND E AND A;
 A AND B AND D AND E AND F AND G;
 
+A OR D OR B OR E OR F OR G OR E OR A;
+A OR B OR D OR E OR F OR G;
+
+# dialect: mysql
+A XOR D XOR B XOR E XOR F XOR G XOR C;
+A XOR B XOR C XOR D XOR E XOR F XOR G;
+
 A AND NOT B AND C AND B;
 FALSE;
 
@@ -297,6 +370,12 @@ x * (1 - y);
 (((x % 20) = 0) = TRUE);
 ((x % 20) = 0) = TRUE;
 
+ANY(t.value);
+ANY(t.value);
+
+SELECT (ARRAY_AGG(foo))[1];
+SELECT (ARRAY_AGG(foo))[1];
+
 --------------------------------------
 -- Literals
 --------------------------------------
@@ -427,7 +506,7 @@ FALSE;
 TRUE;
 
 date '1998-12-01' - interval x day;
-CAST('1998-12-01' AS DATE) - INTERVAL x day;
+CAST('1998-12-01' AS DATE) - INTERVAL x DAY;
 
 date '1998-12-01' - interval '90' day;
 CAST('1998-09-02' AS DATE);
@@ -442,13 +521,13 @@ interval '1' year + date '1998-01-01' + 3 * 7 * 4;
 CAST('1999-01-01' AS DATE) + 84;
 
 date '1998-12-01' - interval '90' foo;
-CAST('1998-12-01' AS DATE) - INTERVAL '90' foo;
+CAST('1998-12-01' AS DATE) - INTERVAL '90' FOO;
 
 date '1998-12-01' + interval '90' foo;
-CAST('1998-12-01' AS DATE) + INTERVAL '90' foo;
+CAST('1998-12-01' AS DATE) + INTERVAL '90' FOO;
 
 CAST(x AS DATE) + interval '1' week;
-CAST(x AS DATE) + INTERVAL '1' week;
+CAST(x AS DATE) + INTERVAL '1' WEEK;
 
 CAST('2008-11-11' AS DATETIME) + INTERVAL '5' MONTH;
 CAST('2009-04-11 00:00:00' AS DATETIME);
@@ -456,8 +535,20 @@ CAST('2009-04-11 00:00:00' AS DATETIME);
 datetime '1998-12-01' - interval '90' day;
 CAST('1998-09-02 00:00:00' AS DATETIME);
 
-CAST(x AS DATETIME) + interval '1' week;
-CAST(x AS DATETIME) + INTERVAL '1' week;
+CAST(x AS DATETIME) + interval '1' WEEK;
+CAST(x AS DATETIME) + INTERVAL '1' WEEK;
+
+# dialect: bigquery
+CAST('2023-01-01' AS TIMESTAMP) + INTERVAL 1 DAY;
+CAST('2023-01-02 00:00:00' AS TIMESTAMP);
+
+# dialect: bigquery
+INTERVAL 1 DAY + CAST('2023-01-01' AS TIMESTAMP);
+CAST('2023-01-02 00:00:00' AS TIMESTAMP);
+
+# dialect: bigquery
+CAST('2023-01-02' AS TIMESTAMP) - INTERVAL 1 DAY;
+CAST('2023-01-01 00:00:00' AS TIMESTAMP);
 
 TS_OR_DS_TO_DATE('1998-12-01 00:00:01') - interval '90' day;
 CAST('1998-09-02' AS DATE);
@@ -480,6 +571,15 @@ CAST('2023-01-01 22:00:00' AS DATETIME);
 DATE_ADD(x, 1, 'MONTH');
 DATE_ADD(x, 1, 'MONTH');
 
+DATE_ADD(x, 1);
+DATE_ADD(x, 1, 'DAY');
+
+SELECT 1 WHERE 'foo';
+SELECT 1 WHERE 'foo';
+
+SELECT 1 WHERE NOT 'foo';
+SELECT 1 WHERE NOT 'foo';
+
 --------------------------------------
 -- Comparisons
 --------------------------------------
@@ -696,6 +796,60 @@ x <> 1;
 NOT 1 <> x;
 x = 1;
 
+x > CAST('2024-01-01' AS DATE) OR x > CAST('2023-12-31' AS DATE);
+x > CAST('2023-12-31' AS DATE);
+
+CAST(x AS DATE) > CAST('2024-01-01' AS DATE) OR CAST(x AS DATE) > CAST('2023-12-31' AS DATE);
+CAST(x AS DATE) > CAST('2023-12-31' AS DATE);
+
+FUN() > 0 OR FUN() > 1;
+FUN() > 0;
+
+RAND() > 0 OR RAND() > 1;
+RAND() > 0 OR RAND() > 1;
+
+CAST(1 AS UINT) >= 0;
+TRUE;
+
+CAST(-1 AS TINYINT) <= 0;
+TRUE;
+
+CAST(1 AS INT) = CAST(1 AS UINT);
+TRUE;
+
+CASE WHEN CAST(1 AS TINYINT) = 1 THEN FALSE ELSE TRUE END;
+FALSE;
+
+CAST(1 AS INT) + 1;
+CAST(1 AS INT) + 1;
+
+CAST(CAST(CAST(-1 AS INT) AS INT) AS INT) = -1;
+TRUE;
+
+CAST(-1 AS UINT) <= 0;
+CAST(-1 AS UINT) <= 0;
+
+CAST(-129 AS TINYINT) <= 0;
+CAST(-129 AS TINYINT) <= 0;
+
+CAST(256 AS UINT) >= 0;
+CAST(256 AS UINT) >= 0;
+
+CAST(CAST(CAST(-1 AS INT) AS UINT) AS INT) = 1;
+CAST(CAST(CAST(-1 AS INT) AS UINT) AS INT) = 1;
+
+CAST(x AS TINYINT) = 1;
+CAST(x AS TINYINT) = 1;
+
+CAST(CAST(1 AS INT) AS BOOLEAN) = 1;
+CAST(CAST(1 AS INT) AS BOOLEAN) = 1;
+
+CAST(CAST(CAST(1 AS INT) AS BOOLEAN) AS INT) = 1;
+CAST(CAST(CAST(1 AS INT) AS BOOLEAN) AS INT) = 1;
+
+x > CAST('2023-01-01' AS DATE) AND x < CAST('2023-01-01' AS DATETIME);
+FALSE;
+
 --------------------------------------
 -- COALESCE
 --------------------------------------
@@ -705,6 +859,10 @@ x;
 COALESCE(x, 1) = 2;
 NOT x IS NULL AND x = 2;
 
+# dialect: redshift
+COALESCE(x, 1) = 2;
+COALESCE(x, 1) = 2;
+
 2 = COALESCE(x, 1);
 NOT x IS NULL AND x = 2;
 
@@ -733,7 +891,7 @@ COALESCE(ROW() OVER (), 1) = 1;
 ROW() OVER () = 1 OR ROW() OVER () IS NULL;
 
 a AND b AND COALESCE(ROW() OVER (), 1) = 1;
-a AND b AND (ROW() OVER () = 1 OR ROW() OVER () IS NULL);
+(ROW() OVER () = 1 OR ROW() OVER () IS NULL) AND a AND b;
 
 COALESCE(1, 2);
 1;
@@ -744,6 +902,9 @@ CAST(CAST('2023-01-01' AS TIMESTAMP) AS DATE);
 COALESCE(CAST(NULL AS DATE), x);
 COALESCE(CAST(NULL AS DATE), x);
 
+NOT COALESCE(x, 1) = 2 AND y = 3;
+(x <> 2 OR x IS NULL) AND y = 3;
+
 --------------------------------------
 -- CONCAT
 --------------------------------------
@@ -790,7 +951,7 @@ CONCAT_WS(sep, 'a', 'b');
 CONCAT_WS(sep, 'a', 'b');
 
 'a' || 'b' || x;
-CONCAT('ab', x);
+'ab' || x;
 
 CONCAT(a, b) IN (SELECT * FROM foo WHERE cond);
 CONCAT(a, b) IN (SELECT * FROM foo WHERE cond);
@@ -808,6 +969,10 @@ CAST('2023-12-11' AS DATE);
 DATE_TRUNC(CAST('2023-12-15' AS DATE), WEEK);
 CAST('2023-12-10' AS DATE);
 
+# dialect: bigquery
+DATE_TRUNC(CAST('2023-10-01' AS TIMESTAMP), QUARTER);
+CAST('2023-10-01 00:00:00' AS TIMESTAMP);
+
 # dialect: bigquery
 DATE_TRUNC(CAST('2023-12-16' AS DATE), WEEK);
 CAST('2023-12-10' AS DATE);
@@ -815,41 +980,69 @@ CAST('2023-12-10' AS DATE);
 DATE_TRUNC('year', x) = CAST('2021-01-01' AS DATE);
 x < CAST('2022-01-01' AS DATE) AND x >= CAST('2021-01-01' AS DATE);
 
+# dialect: bigquery
+DATE_TRUNC(x, year) = CAST('2021-01-01' AS TIMESTAMP);
+x < CAST('2022-01-01 00:00:00' AS TIMESTAMP) AND x >= CAST('2021-01-01 00:00:00' AS TIMESTAMP);
+
 DATE_TRUNC('quarter', x) = CAST('2021-01-01' AS DATE);
 x < CAST('2021-04-01' AS DATE) AND x >= CAST('2021-01-01' AS DATE);
 
+# dialect: bigquery
+DATE_TRUNC(x, quarter) = CAST('2021-01-01' AS TIMESTAMP);
+x < CAST('2021-04-01 00:00:00' AS TIMESTAMP) AND x >= CAST('2021-01-01 00:00:00' AS TIMESTAMP);
+
 DATE_TRUNC('month', x) = CAST('2021-01-01' AS DATE);
 x < CAST('2021-02-01' AS DATE) AND x >= CAST('2021-01-01' AS DATE);
 
+# dialect: bigquery
+DATE_TRUNC(x, month) = CAST('2021-01-01' AS TIMESTAMP);
+x < CAST('2021-02-01 00:00:00' AS TIMESTAMP) AND x >= CAST('2021-01-01 00:00:00' AS TIMESTAMP);
+
 DATE_TRUNC('week', x) = CAST('2021-01-04' AS DATE);
 x < CAST('2021-01-11' AS DATE) AND x >= CAST('2021-01-04' AS DATE);
 
 DATE_TRUNC('day', x) = CAST('2021-01-01' AS DATE);
 x < CAST('2021-01-02' AS DATE) AND x >= CAST('2021-01-01' AS DATE);
 
+# dialect: bigquery
+DATE_TRUNC(x, DAY) = CAST('2021-01-01' AS TIMESTAMP);
+x < CAST('2021-01-02 00:00:00' AS TIMESTAMP) AND x >= CAST('2021-01-01 00:00:00' AS TIMESTAMP);
+
 CAST('2021-01-01' AS DATE) = DATE_TRUNC('year', x);
 x < CAST('2022-01-01' AS DATE) AND x >= CAST('2021-01-01' AS DATE);
 
+# dialect: bigquery
+CAST('2021-01-01' AS TIMESTAMP) = DATE_TRUNC(x, year);
+x < CAST('2022-01-01 00:00:00' AS TIMESTAMP) AND x >= CAST('2021-01-01 00:00:00' AS TIMESTAMP);
+
 -- Always false, except for nulls
 DATE_TRUNC('quarter', x) = CAST('2021-01-02' AS DATE);
-DATE_TRUNC('quarter', x) = CAST('2021-01-02' AS DATE);
+DATE_TRUNC('QUARTER', x) = CAST('2021-01-02' AS DATE);
 
 DATE_TRUNC('year', x) <> CAST('2021-01-01' AS DATE);
-x < CAST('2021-01-01' AS DATE) AND x >= CAST('2022-01-01' AS DATE);
+FALSE;
 
 -- Always true, except for nulls
 DATE_TRUNC('year', x) <> CAST('2021-01-02' AS DATE);
-DATE_TRUNC('year', x) <> CAST('2021-01-02' AS DATE);
+DATE_TRUNC('YEAR', x) <> CAST('2021-01-02' AS DATE);
 
 DATE_TRUNC('year', x) <= CAST('2021-01-01' AS DATE);
 x < CAST('2022-01-01' AS DATE);
 
+# dialect: bigquery
+DATE_TRUNC(x, year) <= CAST('2021-01-01' AS TIMESTAMP);
+x < CAST('2022-01-01 00:00:00' AS TIMESTAMP);
+
 DATE_TRUNC('year', x) <= CAST('2021-01-02' AS DATE);
 x < CAST('2022-01-01' AS DATE);
 
 CAST('2021-01-01' AS DATE) >= DATE_TRUNC('year', x);
 x < CAST('2022-01-01' AS DATE);
 
+# dialect: bigquery
+CAST('2021-01-01' AS TIMESTAMP) >= DATE_TRUNC(x, year);
+x < CAST('2022-01-01 00:00:00' AS TIMESTAMP);
+
 DATE_TRUNC('year', x) < CAST('2021-01-01' AS DATE);
 x < CAST('2021-01-01' AS DATE);
 
@@ -872,15 +1065,19 @@ DATE_TRUNC('year', x) > TS_OR_DS_TO_DATE(TS_OR_DS_TO_DATE('2021-01-02'));
 x >= CAST('2022-01-01' AS DATE);
 
 DATE_TRUNC('year', x) > TS_OR_DS_TO_DATE(TS_OR_DS_TO_DATE('2021-01-02', '%Y'));
-DATE_TRUNC('year', x) > TS_OR_DS_TO_DATE(TS_OR_DS_TO_DATE('2021-01-02', '%Y'));
+DATE_TRUNC('YEAR', x) > CAST(STR_TO_TIME('2021-01-02', '%Y') AS DATE);
 
 -- right is not a date
 DATE_TRUNC('year', x) <> '2021-01-02';
-DATE_TRUNC('year', x) <> '2021-01-02';
+DATE_TRUNC('YEAR', x) <> '2021-01-02';
 
 DATE_TRUNC('year', x) IN (CAST('2021-01-01' AS DATE), CAST('2023-01-01' AS DATE));
 (x < CAST('2022-01-01' AS DATE) AND x >= CAST('2021-01-01' AS DATE)) OR (x < CAST('2024-01-01' AS DATE) AND x >= CAST('2023-01-01' AS DATE));
 
+# dialect: bigquery
+DATE_TRUNC(x, year) IN (CAST('2021-01-01' AS TIMESTAMP), CAST('2023-01-01' AS TIMESTAMP));
+(x < CAST('2022-01-01 00:00:00' AS TIMESTAMP) AND x >= CAST('2021-01-01 00:00:00' AS TIMESTAMP)) OR (x < CAST('2024-01-01 00:00:00' AS TIMESTAMP) AND x >= CAST('2023-01-01 00:00:00' AS TIMESTAMP));
+
 -- merge ranges
 DATE_TRUNC('year', x) IN (CAST('2021-01-01' AS DATE), CAST('2022-01-01' AS DATE));
 x < CAST('2023-01-01' AS DATE) AND x >= CAST('2021-01-01' AS DATE);
@@ -894,7 +1091,7 @@ x < CAST('2022-01-01 00:00:00' AS DATETIME) AND x >= CAST('2021-01-01 00:00:00'
 
 -- right side is not a date literal
 DATE_TRUNC('day', x) = CAST(y AS DATE);
-CAST(y AS DATE) = DATE_TRUNC('day', x);
+CAST(y AS DATE) = DATE_TRUNC('DAY', x);
 
 -- nested cast
 DATE_TRUNC('day', x) = CAST(CAST('2021-01-01 01:02:03' AS DATETIME) AS DATE);
@@ -903,6 +1100,9 @@ x < CAST('2021-01-02' AS DATE) AND x >= CAST('2021-01-01' AS DATE);
 TIMESTAMP_TRUNC(x, YEAR) = CAST(CAST('2021-01-01 01:02:03' AS DATE) AS DATETIME);
 x < CAST('2022-01-01 00:00:00' AS DATETIME) AND x >= CAST('2021-01-01 00:00:00' AS DATETIME);
 
+DATE_TRUNC('day', CAST(x AS DATE)) <= CAST('2021-01-01 01:02:03' AS TIMESTAMP);
+CAST(x AS DATE) < CAST('2021-01-02 01:02:03' AS TIMESTAMP);
+
 --------------------------------------
 -- EQUALITY
 --------------------------------------
@@ -961,7 +1161,7 @@ DATE_ADD(DATE_ADD(DATE_TRUNC('week', DATE_SUB(x, 1, DAY)), 1, DAY), 1, YEAR) < C
 x < CAST('2020-01-14' AS DATE);
 
 x - INTERVAL '1' day = CAST(y AS DATE);
-CAST(y AS DATE) = x - INTERVAL '1' day;
+CAST(y AS DATE) = x - INTERVAL '1' DAY;
 
 --------------------------------------
 -- Constant Propagation
@@ -1082,3 +1282,33 @@ CASE WHEN x = y THEN z END;
 
 CASE x1 + x2 WHEN x3 THEN x4 WHEN x5 + x6 THEN x7 ELSE x8 END;
 CASE WHEN x3 = (x1 + x2) THEN x4 WHEN (x1 + x2) = (x5 + x6) THEN x7 ELSE x8 END;
+
+--------------------------------------
+-- Simplify STARTSWITH
+--------------------------------------
+STARTS_WITH('foo', 'f');
+TRUE;
+
+STARTS_WITH('foo', 'g');
+FALSE;
+
+STARTS_WITH('', 'f');
+FALSE;
+
+STARTS_WITH('', '');
+TRUE;
+
+STARTS_WITH('foo', '');
+TRUE;
+
+STARTS_WITH(NULL, y);
+STARTS_WITH(NULL, y);
+
+STARTS_WITH(x, y);
+STARTS_WITH(x, y);
+
+STARTS_WITH('x', y);
+STARTS_WITH('x', y);
+
+STARTS_WITH(x, 'y');
+STARTS_WITH(x, 'y');
