@@ -73,6 +73,10 @@ class TestExpressions(unittest.TestCase):
         self.assertEqual(
             exp.DataType.build("int"), exp.DataType(this=exp.DataType.Type.INT, nested=False)
         )
+        self.assertNotEqual(
+            exp.Identifier(this="a", temporary=True),
+            exp.Identifier(this="a"),
+        )
 
     def test_find(self):
         expression = parse_one("CREATE TABLE x STORED AS PARQUET AS SELECT * FROM y")
@@ -1262,3 +1266,7 @@ FROM foo""",
 
         self.assertIsInstance(result, exp.TsOrDsToTime)
         self.assertEqual(result.sql(), "CAST('12:00:00' AS TIME)")
+
+    def test_hash_large_ast(self):
+        expr = parse_one("SELECT 1 UNION ALL " * 3000 + "SELECT 1")
+        assert expr == expr
