@@ -42,6 +42,22 @@ class TestLineage(unittest.TestCase):
         self.assertEqual(downstream.source_name, "y")
         self.assertGreater(len(node.to_html()._repr_html_()), 1000)
 
+        # test that sql is not modified
+        sql = "SELECT a FROM x"
+        ast = sqlglot.parse_one(sql)
+        node = lineage("a", ast)
+        self.assertEqual(ast.sql(), sql)
+
+        # test that sources are not modified
+        ast = sqlglot.parse_one(sql)
+
+        source_sql = "SELECT a FROM y"
+        source = sqlglot.parse_one(source_sql)
+
+        node = lineage("a", ast, sources={"x": source})
+
+        self.assertEqual(source.sql(), source_sql)
+
     def test_lineage_sql_with_cte(self) -> None:
         node = lineage(
             "a",
@@ -495,3 +511,215 @@ class TestLineage(unittest.TestCase):
         self.assertEqual(len(node.downstream), 1)
         self.assertEqual(len(node.downstream[0].downstream), 1)
         self.assertEqual(node.downstream[0].downstream[0].name, "t1.x")
+
+    def test_pivot_without_alias(self) -> None:
+        sql = """
+        SELECT 
+            a as other_a
+        FROM (select value,category from sample_data)
+        PIVOT (
+            sum(value)
+            FOR category IN ('a', 'b')
+        );
+        """
+        node = lineage("other_a", sql)
+
+        self.assertEqual(node.downstream[0].name, "_q_0.value")
+        self.assertEqual(node.downstream[0].downstream[0].name, "sample_data.value")
+
+    def test_pivot_with_alias(self) -> None:
+        sql = """
+            SELECT 
+                cat_a_s as other_as
+            FROM sample_data
+            PIVOT (
+                sum(value) as s, max(price)
+                FOR category IN ('a' as cat_a, 'b')
+            )
+        """
+        node = lineage("other_as", sql)
+
+        self.assertEqual(len(node.downstream), 1)
+        self.assertEqual(node.downstream[0].name, "sample_data.value")
+
+    def test_pivot_with_cte(self) -> None:
+        sql = """
+        WITH t as (
+            SELECT 
+                a as other_a
+            FROM sample_data
+            PIVOT (
+                sum(value)
+                FOR category IN ('a', 'b')
+            )
+        )
+        select other_a from t
+        """
+        node = lineage("other_a", sql)
+
+        self.assertEqual(node.downstream[0].name, "t.other_a")
+        self.assertEqual(node.downstream[0].reference_node_name, "t")
+        self.assertEqual(node.downstream[0].downstream[0].name, "sample_data.value")
+
+    def test_pivot_with_implicit_column_of_pivoted_source(self) -> None:
+        sql = """
+        SELECT empid
+        FROM quarterly_sales
+            PIVOT(SUM(amount) FOR quarter IN (
+            '2023_Q1',
+            '2023_Q2',
+            '2023_Q3'))
+        ORDER BY empid;
+        """
+        node = lineage("empid", sql)
+
+        self.assertEqual(node.downstream[0].name, "quarterly_sales.empid")
+
+    def test_pivot_with_implicit_column_of_pivoted_source_and_cte(self) -> None:
+        sql = """
+        WITH t as (
+            SELECT empid
+            FROM quarterly_sales
+            PIVOT(SUM(amount) FOR quarter IN (
+                '2023_Q1',
+                '2023_Q2',
+                '2023_Q3'))
+        )
+        select empid from t
+        """
+        node = lineage("empid", sql)
+
+        self.assertEqual(node.downstream[0].name, "t.empid")
+        self.assertEqual(node.downstream[0].reference_node_name, "t")
+        self.assertEqual(node.downstream[0].downstream[0].name, "quarterly_sales.empid")
+
+    def test_table_udtf_snowflake(self) -> None:
+        lateral_flatten = """
+        SELECT f.value:external_id::string AS external_id
+        FROM database_name.schema_name.table_name AS raw,
+        LATERAL FLATTEN(events) AS f
+        """
+        table_flatten = """
+        SELECT f.value:external_id::string AS external_id
+        FROM database_name.schema_name.table_name AS raw
+        JOIN TABLE(FLATTEN(events)) AS f
+        """
+
+        lateral_node = lineage("external_id", lateral_flatten, dialect="snowflake")
+        table_node = lineage("external_id", table_flatten, dialect="snowflake")
+
+        self.assertEqual(lateral_node.name, "EXTERNAL_ID")
+        self.assertEqual(table_node.name, "EXTERNAL_ID")
+
+        lateral_node = lateral_node.downstream[0]
+        table_node = table_node.downstream[0]
+
+        self.assertEqual(lateral_node.name, "F.VALUE")
+        self.assertEqual(
+            lateral_node.source.sql("snowflake"),
+            "LATERAL FLATTEN(RAW.EVENTS) AS F(SEQ, KEY, PATH, INDEX, VALUE, THIS)",
+        )
+
+        self.assertEqual(table_node.name, "F.VALUE")
+        self.assertEqual(table_node.source.sql("snowflake"), "TABLE(FLATTEN(RAW.EVENTS)) AS F")
+
+        lateral_node = lateral_node.downstream[0]
+        table_node = table_node.downstream[0]
+
+        self.assertEqual(lateral_node.name, "RAW.EVENTS")
+        self.assertEqual(
+            lateral_node.source.sql("snowflake"),
+            "DATABASE_NAME.SCHEMA_NAME.TABLE_NAME AS RAW",
+        )
+
+        self.assertEqual(table_node.name, "RAW.EVENTS")
+        self.assertEqual(
+            table_node.source.sql("snowflake"),
+            "DATABASE_NAME.SCHEMA_NAME.TABLE_NAME AS RAW",
+        )
+
+    def test_pivot_with_subquery(self) -> None:
+        schema = {
+            "loan_ledger": {
+                "product_type": "varchar",
+                "month": "date",
+                "loan_id": "int",
+            }
+        }
+
+        sql = """
+        WITH cte AS (
+            SELECT * FROM (
+                SELECT product_type, month, loan_id 
+                FROM loan_ledger
+            ) PIVOT (
+                COUNT(loan_id) FOR month IN ('2024-10', '2024-11')
+            )
+        )
+        SELECT 
+            cte.product_type AS product_type,
+            cte."2024-10" AS "2024-10"
+        FROM cte
+        """
+
+        node = lineage("product_type", sql, dialect="duckdb", schema=schema)
+        self.assertEqual(node.downstream[0].name, "cte.product_type")
+        self.assertEqual(node.downstream[0].downstream[0].name, "_q_0.product_type")
+        self.assertEqual(
+            node.downstream[0].downstream[0].downstream[0].name, "loan_ledger.product_type"
+        )
+
+        node = lineage('"2024-10"', sql, dialect="duckdb", schema=schema)
+        self.assertEqual(node.downstream[0].name, "cte.2024-10")
+        self.assertEqual(node.downstream[0].downstream[0].name, "_q_0.loan_id")
+        self.assertEqual(node.downstream[0].downstream[0].downstream[0].name, "loan_ledger.loan_id")
+
+    def test_copy_flag(self) -> None:
+        schema = {
+            "x": {
+                "a": "int",
+            },
+        }
+
+        query = sqlglot.parse_one("SELECT a FROM z")
+        sources = {
+            "y": sqlglot.parse_one("SELECT * FROM x"),
+            "z": sqlglot.parse_one("SELECT * FROM y"),
+        }
+
+        lineage("a", query, schema=schema, sources=sources, copy=False)
+
+        self.assertEqual(sources["y"].sql(), "SELECT * FROM x")
+        self.assertEqual(sources["z"].sql(), "SELECT * FROM y")
+        self.assertEqual(
+            query.sql(),
+            "SELECT z.a AS a FROM (SELECT y.a AS a FROM (SELECT x.a AS a FROM x AS x) AS y /* source: y */) AS z /* source: z */",
+        )
+
+        query = sqlglot.parse_one("SELECT a FROM z")
+        sources = {
+            "y": sqlglot.parse_one("SELECT * FROM x"),
+            "z": sqlglot.parse_one("SELECT * FROM y"),
+        }
+
+        lineage("a", query, schema=schema, sources=sources, copy=True)
+
+        self.assertEqual(sources["y"].sql(), "SELECT * FROM x")
+        self.assertEqual(sources["z"].sql(), "SELECT * FROM y")
+        self.assertEqual(query.sql(), "SELECT a FROM z")
+
+        query = sqlglot.parse_one("SELECT a FROM z")
+        sources = {
+            "y": sqlglot.parse_one("SELECT * FROM x"),
+            "z": sqlglot.parse_one("SELECT * FROM y"),
+        }
+
+        query = sqlglot.parse_one("SELECT a FROM x")
+        lineage("a", query, schema=schema, copy=False)
+
+        self.assertEqual(query.sql(), "SELECT x.a AS a FROM x AS x")
+
+        query = sqlglot.parse_one("SELECT a FROM x")
+        lineage("a", query, schema=schema, copy=True)
+
+        self.assertEqual(query.sql(), "SELECT a FROM x")
